package com.shcd.tcpais.config;

import com.shcd.tcpais.handler.NettyTcpServerHandler;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.string.StringDecoder;
import io.netty.handler.codec.string.StringEncoder;
import io.netty.handler.timeout.IdleStateHandler;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * 类名:@Name - NettyServerConfig
 * 功能描述:@Function Description<>
 * 日期:@Date - 2025/1/17 10:09
 */
@Configuration
public class NettyServerConfig {


    @Bean(destroyMethod = "shutdownGracefully")
    public EventLoopGroup bossGroup() {
        return new NioEventLoopGroup(1);
    }

    @Bean(destroyMethod = "shutdownGracefully")
    public EventLoopGroup workerGroup() {
        return new NioEventLoopGroup();
    }

    @Bean
    public ServerBootstrap serverBootstrap(EventLoopGroup bossGroup, EventLoopGroup workerGroup) {
        ServerBootstrap b = new ServerBootstrap();
        b.group(bossGroup, workerGroup)
                .channel(NioServerSocketChannel.class)
                .option(ChannelOption.SO_BACKLOG, 128)
                .childHandler(new ChannelInitializer<SocketChannel>() {
                    @Override
                    public void initChannel(SocketChannel ch) throws Exception {
                        ch.pipeline()
                                .addLast(new IdleStateHandler(0, 0, 10)) // 心跳检测，10 秒未读则触发事件
                                .addLast(new StringDecoder())
                                .addLast(new StringEncoder())
                                .addLast(new NettyTcpServerHandler());
                    }
                });
        return b;
    }

    @Bean
    public ChannelFuture serverChannelFuture(ServerBootstrap serverBootstrap) throws Exception {
        ChannelFuture f = serverBootstrap.bind(8086).sync();
        return f;
    }

}
