package com.shcd.tcpais.handler;

import com.shcd.tcpais.entity.BoatInfoEntity;
import com.shcd.tcpais.service.AisAnalysisService;
import com.shcd.tcpais.util.AisAnalysisUtil;
import com.shcd.tcpais.service.impl.AisAnalysisServiceImpl;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.handler.timeout.IdleState;
import io.netty.handler.timeout.IdleStateEvent;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

/**
 * 类名:@Name - NettyTcpServerHandler
 * 功能描述:@Function Description<>
 * 日期:@Date - 2025/1/17 10:11
 */
@Slf4j
@Component
public class NettyTcpServerHandler extends ChannelInboundHandlerAdapter {

    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        if (evt instanceof IdleStateEvent) {
            IdleStateEvent event = (IdleStateEvent) evt;
            if (event.state() == IdleState.READER_IDLE) {
                // 触发了读空闲事件，可认为客户端断开连接，关闭通道
                log.info("客户端断开连接，关闭通道.");
                ctx.close();
            }
        } else {
            super.userEventTriggered(ctx, evt);
        }
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        System.out.println("客户端断开, 尝试重新连接...");
        ctx.channel().eventLoop().schedule(() -> {
            try {
                // 重新连接的逻辑
                log.info("正在连接");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }, 10, TimeUnit.SECONDS);
        super.channelInactive(ctx);
    }


    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        String request = (String) msg;
        log.info("解析前: " + request);
        BoatInfoEntity boatInfoEntity = new AisAnalysisUtil().explanation(request);
        AisAnalysisService aisAnalysisService = new AisAnalysisServiceImpl();
        aisAnalysisService.saveAisAnalysis(boatInfoEntity);

    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        cause.printStackTrace();
        ctx.close();
    }
}
